#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;

use geo qw($pi);

my $envelope = 'HTCC';
my $file     = 'HTCC.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

#################################################################


# All dimensions in mm

# None of these components are sensitive.
$detector{"mfield"}      = "no";
$detector{"ncopy"}       = 1;
$detector{"pMany"}       = 1;
$detector{"exist"}       = 1;
$detector{"visible"}     = 1;
$detector{"style"}       = 1;
$detector{"sensitivity"} = "no";
$detector{"hit_type"}    = "";
$detector{"identifiers"} = "";


# General description. Index = mirror, 4 mirrors total.

my $number_of_mirrors = 1;
my $mthick  = 1.0;  # mirrors are 1mm thick

# SEMI Axis
my @axisA   = ( 2907.810    ,  1846.155   ,   1786.859   ,  1728.375  );  # Semiaxis A
#my @axisA   = ( 1907.810    ,  1846.155   ,   1786.859   ,  1728.375  );  # Semiaxis A
my @axisB   = ( 1728.673    ,  1612.998   ,   1497.604   ,  1383.621  );  # Semiaxis B
my @axisC   = ( $axisB[0]   ,  $axisB[1]  ,   $axisB[2]  ,  $axisB[3] );  # Semiaxis C - same as B, it's a ellipsoid of rotation

my @axisAv  = ( $axisA[0] - $mthick ,  $axisA[1] - $mthick  ,   $axisA[2]  - $mthick ,  $axisA[3]- $mthick  );  # Semiaxis A thickness
my @axisBv  = ( $axisB[0] - $mthick ,  $axisB[1] - $mthick  ,   $axisB[2]  - $mthick ,  $axisB[3]- $mthick  );  # Semiaxis B thickness
my @axisCv  = ( $axisC[0] - $mthick ,  $axisC[1] - $mthick  ,   $axisC[2]  - $mthick ,  $axisC[3]- $mthick  );  # Semiaxis C thickness


# The first focal point is the target
# The second focal point position is given below (left mirror).
# For an ellipse, the distance between the two focal points is    d = 2 sqrt(a2-b2)
# http://en.wikipedia.org/wiki/Ellipse

my @fc2x    = (  417.483    ,  462.948    ,  492.379     ,  503.907 );  # focal point 2 x position
my @fc2y    = ( 1558.067    , 1727.744    , 1837.584     , 1880.607 );  # focal point 2 y position
my @fc2z    = (  -62.047    ,  163.333    ,  425.482     ,  707.739 );  # focal point 2 z position


my $nsegments = 12;


# G4 materials
my $mirror_material   = 'G4_Si';

# G4 colors
my @mirror_color      = ('99eeff', 'ee99ff', 'ff44ff', 'ffee55');


my $microgap_width = 0.01;




sub make_mirrors1a
{
 for(my $m = 0; $m < $number_of_mirrors; $m++)
 {
    my $mind                 = $m + 1;
    $detector{"name"}        = "aaa_htcc_Mirror$mind";
    $detector{"mother"}      = "root";
    $detector{"description"} = "High Threshold Cerenkov Mirror $mind - Ellipsoid part 1";
    $detector{"pos"}         = "0*cm 0*cm 0*cm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = $mirror_color[$m];
    $detector{"type"}        = "Ellipsoid";
    $detector{"dimensions"}  = "$axisA[$m]*mm $axisB[$m]*mm $axisC[$m]*mm 0*mm 0*mm";
    $detector{"material"}    = "Air";
    $detector{"material"}    = "Component";

    print_det(\%detector, $file);
 }
#  print rad2deg(phi($fc2x[0], $fc2y[0], $fc2z[0]))."\n";
#  print rad2deg(theta($fc2x[0], $fc2y[0], $fc2z[0]));
}

sub make_mirrors1b
{
 for(my $m = 0; $m < $number_of_mirrors; $m++)
 {
    my $mind                 = $m + 1;
    $detector{"name"}        = "aab_htcc_Mirror$mind";
    $detector{"mother"}      = "root";
    $detector{"description"} = "High Threshold Cerenkov Mirror $mind - Ellipsoid part 2";
    $detector{"pos"}         = "-40*mm 0*cm 0*cm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = $mirror_color[$m];
    $detector{"type"}        = "Ellipsoid";
    $detector{"dimensions"}  = "$axisA[$m]*mm $axisB[$m]*mm $axisC[$m]*mm 0*mm 0*mm";
#    $detector{"dimensions"}  = "$axisAv[$m]*mm $axisBv[$m]*mm $axisCv[$m]*mm 0*mm 0*mm";
    $detector{"material"}    = "Air";
    $detector{"material"}    = "Component";

    print_det(\%detector, $file);

 }
}

sub make_mirrors1c
{
 for(my $m = 0; $m < $number_of_mirrors; $m++)
 {
    my $mind                 = $m + 1;
    $detector{"name"}        = "aac_htcc_Mirror$mind";
    $detector{"mother"}      = "root";
    $detector{"description"} = "High Threshold Cerenkov Mirror $mind - Ellipsoid part1 - part2";
    $detector{"pos"}         = "0*cm 0*cm 0*cm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = $mirror_color[$m];
    $detector{"type"}        = "Operation: aaa_htcc_Mirror$mind - aab_htcc_Mirror$mind";
    $detector{"dimensions"}  = "0";
    $detector{"material"}    = "Component";

    print_det(\%detector, $file);
 }
}

make_mirrors1a();
make_mirrors1b();
make_mirrors1c();

# Tube used for subtraction
my $tune_R         = 3000.0;
my $tube_z         = 3000.0;
my $tube_start_phi = 15.0;
my $tube_phi_span  = 360.0 - 30.0;


sub make_sub_tube
{
 $detector{"name"}        = "aad_htcc_tube";
 $detector{"mother"}      = "root";
 $detector{"description"} = "Tube 1 used to cut the mirrors";
 $detector{"pos"}         = "0*mm 0*mm 0*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ffffff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "0*mm $tune_R*mm $tube_z*mm $tube_start_phi*deg $tube_phi_span*deg";
 $detector{"material"}    = "Air";
 $detector{"material"}    = "Component";

 print_det(\%detector, $file);

}



sub make_mirrors1e
{
 for(my $m = 0; $m < $number_of_mirrors; $m++)
 {
    my $mind                 = $m + 1;
    $detector{"name"}        = "aae_htcc_Mirror$mind";
    $detector{"mother"}      = "root";
    $detector{"description"} = "High Threshold Cerenkov Mirror $mind - Half Ellipsoid";
    $detector{"pos"}         = "0*cm 0*cm 0*cm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = $mirror_color[$m];
    $detector{"type"}        = "Operation: aaa_htcc_Mirror$mind - aad_htcc_tube";
    $detector{"dimensions"}  = "0";
    $detector{"material"}    = "Air";
    $detector{"material"}    = "Component";

    print_det(\%detector, $file);
 }
}


make_sub_tube();
make_mirrors1e();



# Box used for subtraction
my $box_x = 3000.0;
my $box_y = 3000.0;
my $box_z = 3000.0;


sub make_sub_box1
{
 $detector{"name"}        = "aad_htcc_box1";
 $detector{"mother"}      = "root";
 $detector{"description"} = "Box 1 used to cut the mirrors in half";
 $detector{"pos"}         = "0*mm 0*mm 3600*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ffffff";
 $detector{"type"}        = "Box";
 $detector{"dimensions"}  = "$box_x*mm $box_y*mm $box_z*mm";
 $detector{"material"}    = "Air";
 $detector{"material"}    = "Component";

 print_det(\%detector, $file);

}
sub make_sub_box2
{
 $detector{"name"}        = "aad_htcc_box2";
 $detector{"mother"}      = "root";
 $detector{"description"} = "Box 1 used to cut the mirrors in half";
 $detector{"pos"}         = "0*mm 0*mm -3600*mm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "ffffff";
 $detector{"type"}        = "Box";
 $detector{"dimensions"}  = "$box_x*mm $box_y*mm $box_z*mm";
 $detector{"material"}    = "Air";
 $detector{"material"}    = "Component";

 print_det(\%detector, $file);

}


sub make_mirrors1f
{
 for(my $m = 0; $m < $number_of_mirrors; $m++)
 {
    my $mind                 = $m + 1;
    $detector{"name"}        = "aaf_htcc_Mirror$mind";
    $detector{"mother"}      = "root";
    $detector{"description"} = "High Threshold Cerenkov Mirror $mind - Half Ellipsoid";
    $detector{"pos"}         = "0*cm 0*cm 0*cm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = $mirror_color[$m];
    $detector{"type"}        = "Operation: aae_htcc_Mirror$mind - aad_htcc_box1";
    $detector{"dimensions"}  = "0";
    $detector{"material"}    = "Air";
    $detector{"material"}    = "Component";

    print_det(\%detector, $file);
 }
}
sub make_mirrors1g
{
 for(my $m = 0; $m < $number_of_mirrors; $m++)
 {
    my $mind                 = $m + 1;
    $detector{"name"}        = "aag_htcc_Mirror$mind";
    $detector{"mother"}      = "root";
    $detector{"description"} = "High Threshold Cerenkov Mirror $mind - Half Ellipsoid";
    $detector{"pos"}         = "0*cm 0*cm 0*cm";
    $detector{"rotation"}    = "0*deg 0*deg 0*deg";
    $detector{"color"}       = $mirror_color[$m];
    $detector{"type"}        = "Operation: aaf_htcc_Mirror$mind - aad_htcc_box2";
    $detector{"dimensions"}  = "0";
    $detector{"material"}    = "Air";
#    $detector{"material"}    = "Component";

    print_det(\%detector, $file);
 }
}


make_sub_box1();
make_sub_box2();
make_mirrors1f();
make_mirrors1g();





















