#!/usr/bin/perl -w

use strict;

use lib ("$ENV{GEMC}/database_io");
use geo;
use geo qw($pi);


my $envelope = 'BST2';
my $file     = 'BST2.txt';

my $rmin      = 1;
my $rmax      = 1000000;

my %detector = ();    # hash (map) that defines the gemc detector
$detector{"rmin"} = $rmin;
$detector{"rmax"} = $rmax;

#################################################################



# All dimensions in mm
#
# Overlap version of the barrel part.
# The tilt angles can be calculated from:
#
#
#
#          | .
#          |   .
#  R2-R1   |     . L
#          |       .
#          |         .
#          |___________  theta
#
#
#   L sin(theta) = R2-R1
#


# General description
my @radius1            = (50.0    ,  88.0    ,  124.0    ,  160.0   );  # Inner radii of barrel SVT - innermost radius R1
my @radius2            = (68.0    , 100.877  ,  134.325  ,  168.580 );  # Inner radii of barrel SVT - outermost radius R2
my @radius             = (55.876  ,  92.203  ,  127.521  ,  163.137 );  # Inner radii of barrel SVT - center
my @number_of_modules  = (1       , 2        , 3         , 3        );  # Number of modules for each layer
my @nsegments          = (10      , 18       , 22        , 27       );  # Number of segments in each layer
my @starting_point     = (-39.219 , -111.319 , -164.787  , -107.07  );  # Z starting_point of the moduli
my @starting_theta     = (-90     , -90      , -90       , -90      );  # Starting angle of the first segment. 360/18/2 layer 3
my @dtheta             = (0       , 0        , 0         , 0        );  # Delta theta
my @ttheta             = (0       , 0        , 0         , 0        );  # Tilt theta



#######################
# Dimensions:
# Each segment is a Box
#######################
my $rohacell_length       = 443.592;                         # Rohacell support length
my $rohacell_height       = 38.00;                           # Rohacell support height
my $roh_carbon_gap        = 5.0;                             # Gap between rohacell and carbon fiber
my $module_length         = 111.625;                         # Module length
my $module_height         = 41.70;                           # Module height
my $carbon_fiber_length   = 433.592;                         # Carbon Fiber support length
my $rohacell_starting     = 227.805 - $carbon_fiber_length;  # Starting point of the rohacell support
my $readout_plate_length  = 41.00;                           # Readout plate length
my $pcboard_length        = 37.00;                           # PC Board length
my $pcboard_height        = 40.00;                           # PC Board height
my $wirebond_length       = 4.50;                            # Wirebond length
my $chip_length           = 5.0;                             # chip length
my $chip_height           = 8.0;                             # chip height
my $chip_displacement     = 8.0;                             # chip displacement from center



# widths
my $silicon_width      = 0.3;       # Silicon sensor width
my $rohacell_width     = 2.0;       # Rohacell width
my $carbon_fiber_width = 0.25;      # Carbon fiber width
my $epoxy_width        = 0.025;     # Epoxy glue width
my $pcboard_width      = 0.5;       # PC Board width
my $chip_width         = 0.5;       # chip width
my $wirebond_width     = 0.3;       # Wirebond width
my $total_width        = 2.0*$silicon_width + $rohacell_width + 2.0*$carbon_fiber_width + 2.0*$epoxy_width ;

for(my $i = 0; $i < 4; $i++)
{
 $dtheta[$i] = 360/$nsegments[$i];
 $ttheta[$i] = asin(($radius2[$i] - $total_width - $radius1[$i]) / $module_height )*180.0/$pi;
 $radius[$i] -=  0.5*$total_width;
}

# G4 materials
my $silicon_material      = 'G4_Si';
my $rohacell_material     = 'Rohacell';
my $carbon_fiber_material = 'CarbonFiber';
my $epoxy_material        = 'Epoxy';
my $pcboard_material      = 'PCBoardM';
my $chip_material         = 'PCBoardM';
my $wirebond_material     = 'svtwirebond';


# G4 colors
my @silicon_color      = ('353540', '404045', '353540');
my $rohacell_color     = 'e10000';
my $carbon_fiber_color = '252020';
my $epoxy_color        = 'e200e1';
my $pcboard_color      = '0000ff';
my $chip_color         = 'fff600';
my $wirebond_color     = 'a5af9a';


my $microgap_width = 0.0008;

sub pos_t
{
 my $R = shift;
 my $l = shift;
 my $s = shift;

 my $theta     = 90 + $starting_theta[$l] + $s*$dtheta[$l] ;
 my $x         = sprintf("%.3f",$R*cos(rad($theta)));
 my $y         = sprintf("%.3f",$R*sin(rad($theta)));
 return "$x*mm $y*mm";
}

sub rot
{
 my $R = shift;
 my $l = shift;
 my $s = shift;

 my $theta_rot = $starting_theta[$l] - $s*$dtheta[$l] + $ttheta[$l];
# if($l == 2) {$theta_rot -= 20;}

 return "0*deg 0*deg $theta_rot*deg";
}

sub pos_t_chip
{
 my $R = shift;
 my $l = shift;
 my $t = shift;
 my $s = shift;

 my $dth = asind($chip_displacement / $R);

 my $theta = 0;

 if($t == 1){ $theta     = 90 - $dth + $starting_theta[$l] + $s*$dtheta[$l] ; }
 if($t == 2){ $theta     = 90 + $dth + $starting_theta[$l] + $s*$dtheta[$l] ; }

 my $x         = sprintf("%.3f",$R*cos(rad($theta)));
 my $y         = sprintf("%.3f",$R*sin(rad($theta)));
 return "$x*mm $y*mm";
}


for(my $l = 0; $l < 4; $l++)
{
 my $layer_no       = $l + 1;

#  place_chip($l, 1, 1);
#  place_chip($l, 1, 2);
 place_wirebond($l, 1);
 place_pcboard($l, 1);
 place_silicon($l, 1);
 place_epoxy($l, 1);
 place_carbon_fiber($l, 1);
 place_rohacell($l);
 place_carbon_fiber($l, 2);
 place_epoxy($l, 2);
 place_silicon($l, 2);
 place_pcboard($l, 2);
 place_wirebond($l, 2);
#  place_chip($l, 2, 1);
#  place_chip($l, 2, 2);
}



# BST is a Tube. SLs are Tubes inside BST

my $bst_ir = 47;
my $bst_or = 168;
my $bst_dz = 481.0/2.0;

my @SL_ir = (49.0, 86,  122, 158);
my @SL_or = (69.0, 102, 136, 170);
my $SL_dz = 480.0/2.0;

make_bst();
make_sl(1);
make_sl(2);
make_sl(3);
make_sl(4);

sub make_bst
{
 $detector{"name"}        = $envelope;
 $detector{"mother"}      = "root";
 $detector{"description"} = "Barrel Silicon Vertex Tracker - Pinwheel Version";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "aaaaff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$bst_ir*mm $bst_or*mm $bst_dz*mm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}


sub make_sl
{
 my $slnumber = shift;
 my $slindex  = $slnumber - 1;

 $detector{"name"}        = "SL_$slnumber";
 $detector{"mother"}      = $envelope;
 $detector{"description"} = "Super Layer $slnumber";
 $detector{"pos"}         = "0*cm 0*cm 0*cm";
 $detector{"rotation"}    = "0*deg 0*deg 0*deg";
 $detector{"color"}       = "aaaaff";
 $detector{"type"}        = "Tube";
 $detector{"dimensions"}  = "$SL_ir[$slindex]*mm $SL_or[$slindex]*mm $SL_dz*mm 0*deg 360*deg";
 $detector{"material"}    = "Air";
 $detector{"mfield"}      = "no";
 $detector{"ncopy"}       = 1;
 $detector{"pMany"}       = 1;
 $detector{"exist"}       = 1;
 $detector{"visible"}     = 0;
 $detector{"style"}       = 0;
 $detector{"sensitivity"} = "no";
 $detector{"hit_type"}    = "";
 $detector{"identifiers"} = "";

 print_det(\%detector, $file);
}



sub place_silicon
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r          = $radius[$l] + $silicon_width/2.0;
    $vname      = "BST_SiV_Layer";
    $descriptio = "Barrel Silicon Tracker, Silicon V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r          = $radius[$l] + 1.5*$silicon_width + 2*$epoxy_width + 2*$carbon_fiber_width + $rohacell_width  + 6*$microgap_width; 
    $vname      = "BST_SiW_Layer";
    $descriptio = "Barrel Silicon Tracker, Silicon W, Layer $layer_no, ";
 }

 for(my $m = 0; $m < $number_of_modules[$l]; $m++)
 {
    my $module_n = $m + 1;

    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
       my $snumber   = cnumber($s, 10);
       my $tpos      = pos_t($r, $l, $s);
       my $z         = $module_length / 2 + $starting_point[$l] + $m*$module_length + $m*$microgap_width;
       my $Dx        = $module_height / 2;
       my $Dy        = $silicon_width / 2;
       my $Dz        = $module_length / 2;
       my $segment_n = $s + 1;

       $detector{"name"}        = "$vname$layer_no\_Module$module_n\_Segment$snumber";
       $detector{"mother"}      = "SL_$layer_no";
       $detector{"description"} = "$descriptio Module $module_n, Segment $segment_n";
       $detector{"pos"}         = "$tpos $z*mm";
       $detector{"rotation"}    = rot($r, $l, $s);
       $detector{"color"}       = $silicon_color[$m];
       $detector{"type"}        = "Box";
       $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
       $detector{"material"}    = $silicon_material;
       $detector{"mfield"}      = "no";
       $detector{"ncopy"}       = $s + 1;
       $detector{"pMany"}       = 1;
       $detector{"exist"}       = 1;
       $detector{"visible"}     = 1;
       $detector{"style"}       = 1;
       $detector{"sensitivity"} = "BST";
       $detector{"hit_type"}    = "BST";
       $detector{"identifiers"} = "superlayer manual $layer_no type manual $type segment manual $detector{'ncopy'} module manual $module_n strip manual 1";

       print_det(\%detector, $file);
    }
 }
}

sub place_epoxy
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r          = $radius[$l] + $silicon_width + $epoxy_width / 2.0 + $microgap_width;
    $vname      = "BST_Epoxy_V_Layer";
    $descriptio = "Epoxy V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r          =  $radius[$l] + $silicon_width + 1.5*$epoxy_width + 2.0*$carbon_fiber_width + $rohacell_width  + 5*$microgap_width; 
    $vname      = "BST_Epoxy_W_Layer";
    $descriptio = "Epoxy W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $module_height / 2;
    my $Dy        = $epoxy_width / 2;
    my $Dz        = ( $number_of_modules[$l]*$module_length + $number_of_modules[$l]*$microgap_width ) / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $Dz + $starting_point[$l] ;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $epoxy_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $epoxy_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}

sub place_carbon_fiber
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r           = $radius[$l] + $silicon_width + $epoxy_width + $carbon_fiber_width / 2.0 + 2.0*$microgap_width;
    $vname       = "BST_CarbonFiber_V_L";
    $descriptio  = "Epoxy V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r           = $radius[$l] + $silicon_width + $epoxy_width + 1.5*$carbon_fiber_width + $rohacell_width  + 4.0*$microgap_width;
    $vname       = "BST_CarbonFiber_W_L";
    $descriptio  = "Epoxy W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $module_height / 2;
    my $Dy        = $carbon_fiber_width / 2;
    my $Dz        = $carbon_fiber_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         =  $rohacell_starting + $roh_carbon_gap + $Dz;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $carbon_fiber_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $carbon_fiber_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}

sub place_pcboard
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r           = $radius[$l] - $pcboard_width / 2.0 - $microgap_width ;
    $vname       = "BST_PCBoard_V_L";
    $descriptio  = "PC Board V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r           = $radius[$l] + 2*$silicon_width + 2*$epoxy_width + 2*$carbon_fiber_width + $rohacell_width  + $pcboard_width / 2.0 + 7.0*$microgap_width ; 
    $vname       = "BST_PCBoard_W_L";
    $descriptio  = "PC Board W, Layer $layer_no, ";
 }

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $pcboard_height / 2.0;
    my $Dy        = $pcboard_width  / 2.0;
    my $Dz        = $pcboard_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         =  $starting_point[$l] - $microgap_width - $Dz - $wirebond_length;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $pcboard_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $pcboard_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}

sub place_rohacell
{
 my $l    = shift;
 my $layer_no       = $l + 1;
 my $r = $radius[$l] + $silicon_width + $epoxy_width + $carbon_fiber_width + $rohacell_width / 2.0 + 3*$microgap_width;

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $rohacell_height / 2;
    my $Dy        = $rohacell_width / 2;
    my $Dz        = $rohacell_length / 2.0 ;
    my $tpos      = pos_t($r, $l, $s);
    my $z         = $rohacell_starting + $Dz ;
    my $segment_n = $s + 1;

    $detector{"name"}        = "BST_Rohacell_L$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "Rohacell support, Layer $layer_no, Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $rohacell_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $rohacell_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}


sub place_wirebond
{
 my $l          = shift;
 my $type       = shift;
 my $layer_no   = $l + 1;
 my $r          = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r          =  $radius[$l] - $pcboard_width  - $wirebond_width / 2.0 - $microgap_width ;
    $vname      = "BST_Wirebond_V_Layer";
    $descriptio = "Barrel Silicon Tracker, Wirebond V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r          = $radius[$l] + 2*$silicon_width + 2*$epoxy_width + 2*$carbon_fiber_width + $rohacell_width  + $pcboard_width + $wirebond_width / 2.0 + 7.0*$microgap_width ;  
    $vname      = "BST_Wirebond_W_Layer";
    $descriptio = "Barrel Silicon Tracker, Wirebond W, Layer $layer_no, ";
 }

 for(my $m = 0; $m < $number_of_modules[$l]; $m++)
 {
    my $module_n = $m + 1;
    for(my $s = 0; $s < $nsegments[$l] ; $s++)
    {
       my $snumber   = cnumber($s, 10);
       my $Dx        = $pcboard_height  / 2.0;
       my $Dy        = $wirebond_width  / 2.0;
       my $Dz        = $wirebond_length / 2.0;
       my $tpos      = pos_t($r, $l, $s);
       my $z         = $starting_point[$l] + $m*$module_length - $Dz;
       my $segment_n = $s + 1;

       $detector{"name"}        = "$vname$layer_no\_Module$module_n\_Segment$snumber";
       $detector{"mother"}      = "SL_$layer_no";
       $detector{"description"} = "$descriptio Module $module_n, Segment $segment_n";
       $detector{"type"}        = "Box";
       $detector{"pos"}         = "$tpos $z*mm";
       $detector{"rotation"}    = rot($r, $l, $s);
       $detector{"color"}       = $wirebond_color;
       $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
       $detector{"material"}    = $silicon_material;
       $detector{"mfield"}      = "no";
       $detector{"ncopy"}       = $s + 1;
       $detector{"pMany"}       = 1;
       $detector{"exist"}       = 1;
       $detector{"visible"}     = 1;
       $detector{"style"}       = 1;
       $detector{"sensitivity"} = "no";
       $detector{"hit_type"}    = "";
       $detector{"identifiers"} = "";

       print_det(\%detector, $file);
    }
 }
}

sub place_chip
{
 my $l          = shift;
 my $type       = shift;
 my $left_rigth = shift;
 my $layer_no   = $l + 1;
 my $r1         = 0;
 my $vname      = 0;
 my $descriptio = 0;

 if($type == 1)
 {
    $r1           = $radius[$l] - $pcboard_width - $chip_width / 2.0 - 4.0*$microgap_width ;
    $vname       = "BST_chip_$left_rigth\_V_L";
    $descriptio  = "chip V, Layer $layer_no, ";
 }

 if($type == 2)
 {
    $r1          = $radius[$l] + 2*$silicon_width + 2*$epoxy_width + 2*$carbon_fiber_width + $rohacell_width  + $pcboard_width + $chip_width / 2.0 + 10.0*$microgap_width ; 
    $vname       = "BST_chip_$left_rigth\_W_L";
    $descriptio  = "chip W, Layer $layer_no, ";
 }
 my $r           = sqrt($r1 * $r1 + $chip_displacement * $chip_displacement);

 for(my $s = 0; $s < $nsegments[$l] ; $s++)
 {
    my $snumber   = cnumber($s, 10);
    my $Dx        = $chip_height / 2.0;
    my $Dy        = $chip_width  / 2.0;
    my $Dz        = $chip_length / 2.0 ;
    my $tpos      = pos_t_chip($r, $l, $left_rigth, $s);
    my $z         = $starting_point[$l] - $microgap_width - $Dz - $wirebond_length;
    my $segment_n = $s + 1;

    $detector{"name"}        = "$vname$layer_no\_Segment$snumber";
    $detector{"mother"}      = "SL_$layer_no";
    $detector{"description"} = "$descriptio Segment $segment_n";
    $detector{"pos"}         = "$tpos $z*mm";
    $detector{"rotation"}    = rot($r, $l, $s);
    $detector{"color"}       = $chip_color;
    $detector{"type"}        = "Box";
    $detector{"dimensions"}  = "$Dx*mm $Dy*mm $Dz*mm";
    $detector{"material"}    = $pcboard_material;
    $detector{"mfield"}      = "no";
    $detector{"ncopy"}       = $s + 1;
    $detector{"pMany"}       = 1;
    $detector{"exist"}       = 1;
    $detector{"visible"}     = 1;
    $detector{"style"}       = 1;
    $detector{"sensitivity"} = "no";
    $detector{"hit_type"}    = "";
    $detector{"identifiers"} = "";

    print_det(\%detector, $file);
 }
}











